"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentSetBuilder = void 0;
const path = require("path");
const source_deploy_retrieve_1 = require("@salesforce/source-deploy-retrieve");
const core_1 = require("@salesforce/core");
class ComponentSetBuilder {
    /**
     * Builds a ComponentSet that can be used for source conversion,
     * deployment, or retrieval, using all specified options.
     *
     * @see https://github.com/forcedotcom/source-deploy-retrieve/blob/develop/src/collections/componentSet.ts
     *
     * @param options: options for creating a ComponentSet
     */
    static async build(options) {
        var _a, _b;
        const logger = core_1.Logger.childFromRoot('createComponentSet');
        const csAggregator = [];
        const { directory, manifest, metadata, packagenames, apiversion } = options;
        if (directory) {
            logger.debug(`Building ComponentSet from directory: ${directory.toString()}`);
            directory.forEach((filepath) => {
                if (core_1.fs.fileExistsSync(filepath)) {
                    csAggregator.push(...source_deploy_retrieve_1.ComponentSet.fromSource(path.resolve(filepath)));
                }
                else {
                    throw new core_1.SfdxError(`The sourcepath "${filepath}" is not a valid source file path.`);
                }
            });
        }
        // Return empty ComponentSet and use packageNames in the library via `.retrieve` options
        if (packagenames) {
            logger.debug(`Building ComponentSet for packagenames: ${packagenames.toString()}`);
            csAggregator.push(...new source_deploy_retrieve_1.ComponentSet([]));
        }
        // Resolve manifest with source in package directories.
        if (manifest) {
            logger.debug(`Building ComponentSet from manifest: ${manifest.manifestPath}`);
            const directoryPaths = manifest.directoryPaths;
            logger.debug(`Searching in packageDir: ${directoryPaths.join(', ')} for matching metadata`);
            const compSet = await source_deploy_retrieve_1.ComponentSet.fromManifest({
                manifestPath: manifest.manifestPath,
                resolveSourcePaths: manifest.directoryPaths,
            });
            csAggregator.push(...compSet);
        }
        // Resolve metadata entries with source in package directories.
        if (metadata) {
            logger.debug(`Building ComponentSet from metadata: ${metadata.metadataEntries.toString()}`);
            // Build a Set of metadata entries
            const filter = new source_deploy_retrieve_1.ComponentSet();
            metadata.metadataEntries.forEach((entry) => {
                const splitEntry = entry.split(':');
                filter.add({
                    type: splitEntry[0],
                    fullName: splitEntry.length === 1 ? '*' : splitEntry[1],
                });
            });
            const directoryPaths = metadata.directoryPaths;
            logger.debug(`Searching for matching metadata in directories: ${directoryPaths.join(', ')}`);
            const fromSource = source_deploy_retrieve_1.ComponentSet.fromSource({ fsPaths: directoryPaths, include: filter });
            // If no matching metadata is found, default to the original component set
            const finalized = fromSource.size > 0 ? fromSource : filter;
            csAggregator.push(...finalized);
        }
        const componentSet = new source_deploy_retrieve_1.ComponentSet(csAggregator);
        // This is only for debug output of matched files based on the command flags.
        // It will log up to 20 file matches.
        if (logger.debugEnabled && componentSet.size) {
            logger.debug(`Matching metadata files (${componentSet.size}):`);
            const components = componentSet.getSourceComponents().toArray();
            for (let i = 0; i < componentSet.size; i++) {
                if ((_a = components[i]) === null || _a === void 0 ? void 0 : _a.content) {
                    logger.debug(components[i].content);
                }
                else if ((_b = components[i]) === null || _b === void 0 ? void 0 : _b.xml) {
                    logger.debug(components[i].xml);
                }
                if (i > 18) {
                    logger.debug(`(showing 20 of ${componentSet.size} matches)`);
                    break;
                }
            }
        }
        if (apiversion) {
            componentSet.apiVersion = apiversion;
        }
        return componentSet;
    }
}
exports.ComponentSetBuilder = ComponentSetBuilder;
//# sourceMappingURL=componentSetBuilder.js.map